# -*- coding: utf-8 -*-
from plone import api
from plone.login.browser.login_help import RequestResetPassword
from plone.login.browser.login_help import RequestUsername
from plone.login.testing import PLONE_LOGIN_FUNCTIONAL_TESTING
from plone.login.testing import PLONE_LOGIN_INTEGRATION_TESTING
from plone.testing.z2 import Browser
from zope.component import getMultiAdapter

import transaction
import unittest


class TestLoginHelp(unittest.TestCase):
    layer = PLONE_LOGIN_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer['request']
        self.portal = self.layer['portal']

    def test_view(self):
        view = getMultiAdapter((self.portal, self.request), name='login-help')
        self.assertTrue(view())

    def test_view_form(self):
        form = getMultiAdapter((self.portal, self.request), name='login-help')
        self.assertEqual(form.subforms, [])
        form.update()
        self.assertEqual(len(form.subforms), 2)
        reset_password = form.subforms[0]
        self.assertTrue(isinstance(reset_password, RequestResetPassword))
        self.assertTrue(reset_password())
        request_username = form.subforms[1]
        self.assertTrue(isinstance(request_username, RequestUsername))
        self.assertTrue(request_username())

    def test_view_form_with_emaillogin(self):
        api.portal.set_registry_record('plone.use_email_as_login', True)
        form = getMultiAdapter((self.portal, self.request), name='login-help')
        self.assertEqual(form.subforms, [])
        form.update()
        self.assertEqual(len(form.subforms), 1)
        reset_password = form.subforms[0]
        self.assertTrue(isinstance(reset_password, RequestResetPassword))
        self.assertTrue(reset_password())
        self.assertTrue(form())

    def test_request_reset_password(self):
        form = getMultiAdapter((self.portal, self.request), name='login-help')
        form.update()
        reset_password = form.subforms[0]
        reset_password.handleResetPassword(reset_password, None)
        # the field reset_password is required
        self.assertEqual(reset_password.status, u'There were some errors.')
        # reset error message
        reset_password.status = ''

        self.request['form.widgets.reset_password'] = u'test'
        reset_password.handleResetPassword(reset_password, None)
        self.assertEqual(reset_password.status, '')
        self.assertEqual(len(self.portal.MailHost.messages), 0)
        # no mail was sent since the user does not exist
        self.request['form.widgets.reset_password'] = u'test'

        member = api.user.get('test_user_1_')
        email = 'foo@plone.org'
        member.setMemberProperties({'email': email})
        self.request['form.widgets.reset_password'] = u'test_user_1_'
        reset_password.handleResetPassword(reset_password, None)
        self.assertEqual(reset_password.status, '')
        self.assertEqual(len(self.portal.MailHost.messages), 1)
        message = self.portal.MailHost.messages[0]
        self.assertIn('To: foo@plone.org', message)
        self.assertIn('http://nohost/plone/passwordreset/', message)


class TestLoginHelpFunctional(unittest.TestCase):

    layer = PLONE_LOGIN_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.browser = Browser(self.layer['app'])

    def test_login_help_request_password_reset(self):
        self.browser.open('http://nohost/plone/login')
        self.browser.getLink('Get help').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/@@login-help')

        member = api.user.get('test_user_1_')
        email = 'foo@plone.org'
        member.setMemberProperties({'email': email})
        transaction.commit()
        # validaton error of empty required field
        self.browser.getControl(name='form.buttons.reset').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/@@login-help')

        self.browser.getControl(name='form.widgets.reset_password').value = 'nonexistinguser'  # noqa: E501
        self.browser.getControl(name='form.buttons.reset').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/@@login-help')
        # message appears even though no email was sent
        self.assertIn(
        'An email has been sent with instructions on how to reset your password.', self.browser.contents)  # noqa: E501
        self.assertEqual(len(self.portal.MailHost.messages), 0)

        self.browser.getControl(
            name='form.widgets.reset_password').value = 'test_user_1_'
        self.browser.getControl(name='form.buttons.reset').click()
        self.assertIn(
            'An email has been sent with instructions on how to reset your password.', self.browser.contents)  # noqa: E501
        # message was actually sent
        self.assertEqual(len(self.portal.MailHost.messages), 1)

    def test_login_help_request_username(self):
        self.browser.open('http://nohost/plone/login')
        self.browser.getLink('Get help').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/@@login-help')

        member = api.user.get('test_user_1_')
        email = 'foo@plone.org'
        member.setMemberProperties({'email': email})
        transaction.commit()

        # validaton error of empty required field
        self.browser.getControl(name='form.buttons.get_username').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/@@login-help')
        self.assertIn('missing', self.browser.contents)

        self.browser.getControl(name='form.widgets.recover_username').value = 'foo@plone.org'  # noqa: E501
        self.browser.getControl(name='form.buttons.get_username').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/@@login-help')
        # email was sent
        self.assertIn(
            'email has been sent with your username.', self.browser.contents)
        self.assertEqual(len(self.portal.MailHost.messages), 1)
        message = self.portal.MailHost.messages[0]
        self.assertIn('To: foo@plone.org', message)
        self.assertIn('Your username is: test_user_1_', message)

        self.browser.getControl(
            name='form.widgets.recover_username').value = 'noemail'
        self.browser.getControl(name='form.buttons.get_username').click()
        self.assertIn('specified email is not valid.', self.browser.contents)
        # no new message was sent
        self.assertEqual(len(self.portal.MailHost.messages), 1)

        self.browser.getControl(name='form.widgets.recover_username').value = 'bar@plone.org'  # noqa: E501
        self.browser.getControl(name='form.buttons.get_username').click()
        # no new message was sent
        self.assertIn(
            'email has been sent with your username.', self.browser.contents)
        self.assertEqual(len(self.portal.MailHost.messages), 1)

        api.user.create(
            username='another_user_same_email',
            email='foo@plone.org',
            password='password1',
            roles=('Member',),
        )
        transaction.commit()
        self.browser.getControl(name='form.widgets.recover_username').value = 'foo@plone.org'  # noqa: E501
        self.browser.getControl(name='form.buttons.get_username').click()
        # no new message was sent
        self.assertIn(
            'email has been sent with your username.', self.browser.contents)
        self.assertEqual(len(self.portal.MailHost.messages), 1)

        api.user.create(
            username='next_user_new_email',
            email='bar@plone.org',
            password='password1',
            roles=('Member',),
        )
        transaction.commit()
        self.browser.getControl(name='form.widgets.recover_username').value = 'bar@plone.org'  # noqa: E501
        self.browser.getControl(name='form.buttons.get_username').click()
        # a message was sent
        self.assertIn(
            'email has been sent with your username.', self.browser.contents)
        self.assertEqual(len(self.portal.MailHost.messages), 2)



# -*- coding: utf-8 -*-
from plone.app.z3cform.interfaces import IPloneFormLayer
from plone.login.testing import PLONE_LOGIN_INTEGRATION_TESTING
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
from zope.interface import alsoProvides

import re
import unittest


FORM_ID = 'login'


class TestLoginForm(unittest.TestCase):

    layer = PLONE_LOGIN_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.mt = getToolByName(self.portal, 'portal_membership')
        # suitable for testing z3c.form views
        alsoProvides(self.request, IPloneFormLayer)

    def test_login_view(self):
        view = getMultiAdapter((self.portal, self.request), name='login')
        self.assertTrue(view())

    def _setup_authenticator_request(self):
        self.request.set('REQUEST_METHOD', 'POST')
        authenticator = getMultiAdapter(
            (self.portal, self.request),
            name=u'authenticator'
        )
        html = authenticator.authenticator()
        token = re.search('value="(.*)"', html).groups()[0]
        self.request.set('_authenticator', token)

    def test_form_update(self):
        self._setup_authenticator_request()
        self.request['__ac_name'] = u'test'
        self.request['__ac_password'] = u'secret'
        self.request['form.widgets.came_from'] = [u'']
        form = self.portal.restrictedTraverse(FORM_ID)
        form.update()
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)

    def test_failsafe_login_form(self):
        view = getMultiAdapter(
            (self.portal, self.request),
            name='failsafe_login'
        )
        html = view()
        self.assertNotIn('main-container', html)

    def test_failsafe_login_form_update(self):
        self._setup_authenticator_request()
        self.request['__ac_name'] = u'test'
        self.request['__ac_password'] = u'secret'
        self.request['form.widgets.came_from'] = [u'']
        form = self.portal.restrictedTraverse('failsafe_login')
        form.update()
        data, errors = form.extractData()
        self.assertEqual(len(errors), 0)

    def test_login_external(self):
        registry = self.layer['portal'].portal_registry
        registry['plone.external_login_url'] = 'http://testurl/extlogin'
        form = self.portal.restrictedTraverse('login')
        form()
        self.assertEqual(
            registry['plone.external_login_url'],
            form.request.response.getHeader('Location'),
        )

    def test_login_external_with_params(self):
        registry = self.layer['portal'].portal_registry
        registry['plone.external_login_url'] = 'http://testurl/extlogin'
        self.request['came_from'] = 'foo'
        self.request['next'] = 'bar'
        form = self.portal.restrictedTraverse('login')
        form()
        self.assertIn(
            'came_from=foo',
            form.request.response.getHeader('Location'),
        )
        self.assertIn(
            'next=bar',
            form.request.response.getHeader('Location'),
        )

    def test_failsafe_login_external(self):
        registry = self.layer['portal'].portal_registry
        registry['plone.external_login_url'] = 'http://testurl/extlogin'
        form = self.portal.restrictedTraverse('failsafe_login')
        html = form()
        self.assertIsNotNone(html)
        self.assertEqual(None, form.request.response.getHeader('Location'))
        self.assertNotIn('main-container', html)



# -*- coding: utf-8 -*-
from plone.login.testing import PLONE_LOGIN_INTEGRATION_TESTING
from zope.component import getMultiAdapter

import unittest


class TestViews(unittest.TestCase):

    layer = PLONE_LOGIN_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def test_resetpassword_view(self):
        view = getMultiAdapter((self.portal, self.request),
                               name='initial-login-password-change')
        self.assertTrue(view())

    def test_loggedout_view(self):
        view = getMultiAdapter((self.portal, self.request),
                               name='logged-out')
        self.assertTrue(view())



# -*- coding: utf-8 -*-
from plone.login import MessageFactory as _
from plone.schema.email import Email
from plone.z3cform.interfaces import IWrappedForm
from zope import schema
from zope.interface import Interface


class IRedirectAfterLogin(Interface):
    """ Redirect after login adapters should provide this interface """


class IForcePasswordChange(Interface):
    """ Hook point to customize forcing a password change """


class IInitialLogin(Interface):
    """ Hook point to customize what happens the first time a user logs into
        the site """


class ILogin(Interface):
    """ Login form schema """

    login = schema.TextLine(
        title=_(u'label_log_in', default=u'Log in'),
    )

    password = schema.Password(
        title=_(u'label_password', default=u'Password'),
    )


class ILoginForm(IWrappedForm):
    """ Login form marker interface """


class ILoginFormSchema(Interface):
    """ Login form schema """

    ac_name = schema.TextLine(
        title=_(u'label_login_name', default=u'Login Name'),
        required=True,
    )

    ac_password = schema.Password(
        title=_(u'label_password', default=u'Password'),
        required=True,
    )

    came_from = schema.TextLine(
        title=u'Came From',  # not translated, hidden field
        required=False,
    )


class ILoginHelpForm(IWrappedForm):
    """ Login Help form marker interface """


class ILoginHelpFormSchema(Interface):
    """ Login Help form schema """

    reset_password = schema.TextLine(
        title=_(u'label_pwreset_username', default=u'Username'),
        description=_(
            u'help_pwreset_username',
            default=u'Enter your username '
                    u'or email and we’ll send you a password reset link.',
        ),
        required=True,
    )

    recover_username = Email(
        title=_(u'label_pwreset_email', default=u'Email'),
        description=_(
            u'help_pwreset_email',
            default=u'Enter your email and '
                    u'we’ll send you your username.',
        ),
        required=True,
    )



# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone.app.users.browser.passwordpanel import PasswordPanel
from plone.login import MessageFactory as _
from plone.login.interfaces import IForcePasswordChange
from plone.login.interfaces import IInitialLogin
from plone.login.interfaces import ILoginForm
from plone.login.interfaces import ILoginFormSchema
from plone.login.interfaces import IRedirectAfterLogin
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import ISecuritySchema
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from six.moves.urllib import parse
from z3c.form import button
from z3c.form import field
from z3c.form import form
from z3c.form.interfaces import HIDDEN_MODE
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.interface import implementer


# TODO: Scale down this list now that we've removed a lot of
# templates.
LOGIN_TEMPLATE_IDS = [
    'localhost',
    'logged-out',
    'logged_in',
    'login',
    'login_failed',
    'login_form',
    'login_password',
    'login_success',
    'logout',
    'mail_password',
    'mail_password_form',
    'member_search_results',
    'pwreset_finish',
    'passwordreset',
    'register',
    'registered',
    'require_login',
]


@implementer(ILoginForm)
class LoginForm(form.EditForm):
    """ Implementation of the login form """

    fields = field.Fields(ILoginFormSchema)

    id = 'LoginForm'
    label = _('label_log_in', default=u'Log in')

    ignoreContext = True
    prefix = ''

    def render(self):
        registry = queryUtility(IRegistry)
        ext_login_url = registry['plone.external_login_url']
        if not ext_login_url:
            # normal login
            return self.index()
        #  Handle login on this portal where login is external
        url = ext_login_url
        next_ = self.request.get('next', None)
        portal_url = getToolByName(self.context, 'portal_url')
        if next_ is not None and not portal_url.isURLInPortal(next_):
            next_ = None
        if next_:
            url = '{0}?next={1}'.format(url, next_)
        came_from = self.request.get('came_from')
        if came_from:
            url = '{0}&came_from={1}'.format(url, came_from)
        self.request.response.redirect(url)

    def _get_auth(self):
        try:
            return self.context.acl_users.credentials_cookie_auth
        except AttributeError:
            try:
                return self.context.cookie_authentication
            except AttributeError:
                pass

    def updateWidgets(self):
        auth = self._get_auth()

        if auth:
            fieldname_name = auth.get('name_cookie', '__ac_name')
            fieldname_password = auth.get('pw_cookie', '__ac_password')
        else:
            fieldname_name = '__ac_name'
            fieldname_password = '__ac_password'

        self.fields['ac_name'].__name__ = fieldname_name
        self.fields['ac_password'].__name__ = fieldname_password

        super(LoginForm, self).updateWidgets(prefix='')

        if self.use_email_as_login():
            self.widgets[fieldname_name].label = _(u'label_email',
                                                   default=u'Email')
        self.widgets['came_from'].mode = HIDDEN_MODE
        self.widgets['came_from'].value = self.get_came_from()

    def get_came_from(self):
        came_from = self.request.get('came_from', None)
        if not came_from:
            came_from = self.request.get('HTTP_REFERER', None)
            if not came_from:
                return
        url_tool = getToolByName(self.context, 'portal_url')
        if not url_tool.isURLInPortal(came_from):
            return
        came_from_path = parse.urlparse(came_from)[2]
        for login_template_id in LOGIN_TEMPLATE_IDS:
            if login_template_id in came_from_path:
                return
        return came_from

    def updateActions(self):
        super(LoginForm, self).updateActions()
        self.actions['login'].addClass('context')

    @button.buttonAndHandler(_('Log in'), name='login')
    def handleLogin(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        membership_tool = getToolByName(self.context, 'portal_membership')
        status_msg = IStatusMessage(self.request)
        if membership_tool.isAnonymousUser():
            self.request.response.expireCookie('__ac', path='/')
            if self.use_email_as_login():
                status_msg.addStatusMessage(
                    _(
                        u'Login failed. Both email address and password are '
                        u'case sensitive, check that caps lock is not enabled.'
                    ),
                    'error',
                )
            else:
                status_msg.addStatusMessage(
                    _(
                        u'Login failed. Both login name and password are case '
                        u'sensitive, check that caps lock is not enabled.'
                    ),
                    'error',
                )
            return

        member = membership_tool.getAuthenticatedMember()
        must_change_password = member.getProperty('must_change_password', 0)
        login_time = member.getProperty('login_time', '2000/01/01')
        if not isinstance(login_time, DateTime):
            login_time = DateTime(login_time)
        is_initial_login = login_time == DateTime('2000/01/01')

        membership_tool.loginUser(self.request)
        status_msg.addStatusMessage(
            _(
                u'you_are_now_logged_in',
                default=u'Welcome! You are now logged in.',
            ),
            'info'
        )

        if is_initial_login:
            self.handle_initial_login()

        if must_change_password:
            self.force_password_change()

        came_from = data.get('came_from', None)
        self.redirect_after_login(came_from, is_initial_login)

    def handle_initial_login(self):
        handler = queryMultiAdapter(
            (self.context, self.request),
            IInitialLogin,
        )
        if handler:
            handler()

    def force_password_change(self):
        handler = queryMultiAdapter(
            (self.context, self.request),
            IForcePasswordChange,
        )
        if handler:
            handler()

    def redirect_after_login(self, came_from=None, is_initial_login=False):
        adapter = queryMultiAdapter(
            (self.context, self.request),
            IRedirectAfterLogin
        )
        if adapter:
            came_from = adapter(came_from, is_initial_login)
        if not came_from:
            came_from = self.context.absolute_url()

        self.request.response.redirect(came_from)

    def self_registration_enabled(self):
        registry = queryUtility(IRegistry)
        security_settings = registry.forInterface(
            ISecuritySchema,
            prefix='plone'
        )
        return security_settings.enable_self_reg

    def use_email_as_login(self):
        registry = queryUtility(IRegistry)
        security_settings = registry.forInterface(
            ISecuritySchema,
            prefix='plone'
        )
        return security_settings.use_email_as_login


class FailsafeLoginForm(LoginForm):

    def render(self):
        return self.index()


class RequireLoginView(BrowserView):

    def __call__(self):
        portal_state = getMultiAdapter(
            (self.context, self.request),
            name='plone_portal_state',
        )
        portal = portal_state.portal()
        if portal_state.anonymous():
            url = '{0:s}/login'.format(portal.absolute_url())
            came_from = self.request.get('came_from', None)
            if came_from:
                url += '?came_from={0:s}'.format(parse.quote(came_from))
        else:
            url = '{0:s}/insufficient-privileges'.format(portal.absolute_url())

        self.request.response.redirect(url)


class InsufficientPrivilegesView(BrowserView):

    def request_url(self):
        return self.request.get('came_from')


class InitialLoginPasswordChange(PasswordPanel):

    def render(self):
        return self.index()

    @button.buttonAndHandler(
        _(u'label_change_password', default=u'Change Password'),
        name='reset_passwd',
    )
    def action_reset_passwd(self, action):
        super(InitialLoginPasswordChange, self).action_reset_passwd(
            self, action)
        if not action.form.widgets.errors:
            self.request.response.redirect(self.context.portal_url())


class ForcedPasswordChange(PasswordPanel):

    def render(self):
        return self.index()

    @button.buttonAndHandler(
        _(u'label_change_password', default=u'Change Password'),
        name='reset_passwd',
    )
    def action_reset_passwd(self, action):
        super(ForcedPasswordChange, self).action_reset_passwd(self, action)
        if not action.form.widgets.errors:
            membership_tool = getToolByName(self.context, 'portal_membership')
            member = membership_tool.getAuthenticatedMember()
            member.setProperties(must_change_password=0)
            self.request.response.redirect(self.context.portal_url())



# -*- coding: utf-8 -*-
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.login.browser.login import LoginForm
from plone.login.interfaces import IInitialLogin
from plone.login.interfaces import IRedirectAfterLogin
from plone.login.testing import PLONE_LOGIN_FUNCTIONAL_TESTING
from plone.login.testing import PLONE_LOGIN_INTEGRATION_TESTING
from plone.testing.z2 import Browser
from zope.interface import implementer
from zope.interface import Interface
from zope.publisher.interfaces import IRequest

import unittest


@implementer(IRedirectAfterLogin)
class AfterLoginAdapter(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, came_from=None, is_first_login=False):
        return 'http://nohost/plone/sitemap'


@implementer(IInitialLogin)
class InitialLoginAdapter(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        self.context.foo = 'foo'


class TestCameFromFiltering(unittest.TestCase):
    layer = PLONE_LOGIN_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer['request']
        self.portal = self.layer['portal']
        self.form = LoginForm(self.portal, self.request)

    def test_get_came_from_via_request(self):
        self.assertEqual(self.form.get_came_from(), None)
        url = 'https://nohost/plone/foo-bar'
        self.request['came_from'] = url
        self.assertEqual(self.form.get_came_from(), url)

    def test_external_urls_are_ignored(self):
        url = 'https://example.com/maliciousness'
        self.request['came_from'] = url
        self.assertEqual(self.form.get_came_from(), None)

    def test_login_templates_are_filtered(self):
        url = 'https://nohost/plone/logout'
        self.request['came_from'] = url
        self.assertEqual(self.form.get_came_from(), None)

    def test_referer_is_fallback(self):
        url = 'https://nohost/plone/test'
        self.request['came_from'] = None
        self.request['HTTP_REFERER'] = url
        self.assertEqual(self.form.get_came_from(), url)


class TestRedirectAfterLogin(unittest.TestCase):

    layer = PLONE_LOGIN_FUNCTIONAL_TESTING

    def setUp(self):
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False
        self.portal = self.layer['portal']

    def test_redirect_to_portal_if_no_adapter_nor_came_from(self):
        self.browser.open('http://nohost/plone/login')
        self.browser.getLink('Log in').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/login')

        self.browser.getControl('Login Name').value = TEST_USER_NAME
        self.browser.getControl('Password').value = TEST_USER_PASSWORD
        self.browser.getControl('Log in').click()

        self.assertIn('You are now logged in.', self.browser.contents)
        self.assertEqual(self.browser.url,
                         'http://nohost/plone',
                         'Successful login did not redirect to the homepage '
                         'when came_from was not defined.')

        # Now log out.
        self.browser.getLink('Log out').click()

        self.assertIn('You have been logged out.',
                      self.browser.contents,
                      'Logout status message not displayed.')

    def test_redirect_to_came_from_if_no_adapter_found(self):
        self.browser.open('http://nohost/plone/login')
        self.browser.getLink('Log in').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/login')

        self.browser.getControl('Login Name').value = TEST_USER_NAME
        self.browser.getControl('Password').value = TEST_USER_PASSWORD
        self.browser.getControl(name='came_from').value = \
            'http://nohost/plone/contact-info'

        self.browser.getControl('Log in').click()

        self.assertIn('You are now logged in.', self.browser.contents)
        self.assertEqual(self.browser.url,
                         'http://nohost/plone/contact-info',
                         'Successful login did not redirect to the came_from.')

        # Now log out.
        self.browser.getLink('Log out').click()

        self.assertIn('You have been logged out.',
                      self.browser.contents,
                      'Logout status message not displayed.')

    def test_redirect_to_adapter_result(self):
        # Register our redirect adapter
        from zope.component import getGlobalSiteManager
        gsm = getGlobalSiteManager()
        gsm.registerAdapter(AfterLoginAdapter,
                            (Interface, IRequest))

        self.browser.open('http://nohost/plone/login')
        self.browser.getLink('Log in').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/login')

        self.browser.getControl('Login Name').value = TEST_USER_NAME
        self.browser.getControl('Password').value = TEST_USER_PASSWORD
        self.browser.getControl(name='came_from').value = \
            'http://nohost/plone/contact-info'

        self.browser.getControl('Log in').click()

        gsm.unregisterAdapter(AfterLoginAdapter,
                              (Interface, IRequest))

        self.assertIn('You are now logged in.', self.browser.contents)
        self.assertEqual(self.browser.url,
                         'http://nohost/plone/sitemap',
                         'Successful login did not use the adapter for '
                         'redirect.')

        # Now log out.
        self.browser.getLink('Log out').click()

        self.assertIn('You have been logged out.',
                      self.browser.contents,
                      'Logout status message not displayed.')

    def test_initiallogin_adapter(self):
        # Register our redirect adapter
        from zope.component import getGlobalSiteManager
        gsm = getGlobalSiteManager()
        gsm.registerAdapter(InitialLoginAdapter,
                            (Interface, IRequest))

        self.browser.open('http://nohost/plone/login')
        self.browser.getLink('Log in').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/login')

        self.browser.getControl('Login Name').value = TEST_USER_NAME
        self.browser.getControl('Password').value = TEST_USER_PASSWORD
        self.browser.getControl(name='came_from').value = \
            'http://nohost/plone/contact-info'

        self.browser.getControl('Log in').click()

        gsm.unregisterAdapter(InitialLoginAdapter,
                              (Interface, IRequest))

        self.assertIn('You are now logged in.', self.browser.contents)
        self.assertEqual(self.browser.url, 'http://nohost/plone/contact-info')
        self.assertEqual(self.portal.foo, 'foo')

        # Now log out.
        self.browser.getLink('Log out').click()

        self.assertIn('You have been logged out.',
                      self.browser.contents,
                      'Logout status message not displayed.')



# -*- coding: utf-8 -*-
from email import message_from_string
from email.Header import Header
from plone.login import MessageFactory as _
from plone.login.interfaces import ILoginHelpForm
from plone.login.interfaces import ILoginHelpFormSchema
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import ISecuritySchema
from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from smtplib import SMTPException
from smtplib import SMTPRecipientsRefused
from z3c.form import button
from z3c.form import field
from z3c.form import form
from zope.component import getUtility
from zope.component.hooks import getSite
from zope.interface import implementer
from zope.i18n import translate

import logging


SEND_USERNAME_TEMPLATE = _(u"mailtemplate_username_info", default=u"""From: {encoded_mail_sender}
To: {email}
Subject: Your username for {portal_url}
Content-Type: text/plain
Precedence: bulk

Dear {fullname},

You requested to be reminded of your username for {portal_url}.
Your username is: {login}


With kind regards,

--

{email_from_name}""")

log = logging.getLogger(__name__)


class RequestResetPassword(form.Form):

    id = 'RequestResetPassword'
    label = u''
    fields = field.Fields(ILoginHelpFormSchema).select('reset_password')
    ignoreContext = True

    render = ViewPageTemplateFile('templates/subform_render.pt')

    def updateWidgets(self):
        super(RequestResetPassword, self).updateWidgets()
        if self.use_email_as_login():
            self.widgets['reset_password'].label = _(
                u'label_email',
                default=u'Email'
            )

    @button.buttonAndHandler(
        _(u'button_pwreset_reset_password', default=u'Reset your password'),
        name='reset'
    )
    def handleResetPassword(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        portal = getSite()
        regtool = getToolByName(portal, 'portal_registration')
        try:
            regtool.mailPassword(data['reset_password'], self.request)
        except ValueError as e:
            # Paranoia Warning!
            # We act as if a message has been sent to prevent probing Plone
            # for valid loginnames. Instead we log the error-message.
            log.info('Error while trying to send a reset-password notice to user {0}: {1}'.format(data['reset_password'], e))  # noqa: E501
            pass

        IStatusMessage(self.request).addStatusMessage(
            _(u'statusmessage_pwreset_password_mail_sent', default=u'An '
              u'email has been sent with instructions on how to reset your '
              u'password.'), 'info')

    def use_email_as_login(self):
        registry = getUtility(IRegistry)
        security_settings = registry.forInterface(
            ISecuritySchema, prefix='plone')
        return security_settings.use_email_as_login


class RequestUsername(form.Form):

    id = 'RequestUsername'
    label = u''
    fields = field.Fields(ILoginHelpFormSchema).select('recover_username')
    ignoreContext = True

    render = ViewPageTemplateFile('templates/subform_render.pt')

    @button.buttonAndHandler(
        _(u'button_pwreset_get_username', default='Get your username'),
        name='get_username'
    )
    def handleGetUsername(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        portal = getSite()
        pas = getToolByName(portal, 'acl_users')
        email = data['recover_username']
        results = pas.searchUsers(email=email, exact_match=True)
        send_email = True
        if not results:
            log.info('No user found for {0}'.format(email))
            send_email = False
        if len(results) > 1:
            log.info('More than one user found for {0}'.format(email))
            send_email = False
        if send_email:
            self.send_username(portal, results[0])

        # Paranoia Warning!
        # Same as with the reset-password form we don't want to allow
        # probing for email-adresses of existing users.
        # Because of this we always act as if that an email has been sent.
        # Instead we log the error-message.
        IStatusMessage(self.request).addStatusMessage(
            _(u'statusmessage_pwreset_username_mail_sent',
                default=u'An email has been sent with your username.'),
            'info'
        )

    def send_username(self, portal, userinfo):
        registry = getUtility(IRegistry)
        encoding = registry.get('plone.email_charset', 'utf-8')
        translated_template = translate(
            SEND_USERNAME_TEMPLATE,
            context=self.request,
        )

        mail_text = translated_template.format(
            email=userinfo['email'],
            portal_url=portal.absolute_url(),
            fullname=userinfo['title'],
            login=userinfo['login'],
            email_from_name=registry['plone.email_from_name'],
            encoded_mail_sender=self.encoded_mail_sender(),
        )
        # The mail headers are not properly encoded we need to extract
        # them and let MailHost manage the encoding.
        if isinstance(mail_text, unicode):
            mail_text = mail_text.encode(encoding)
        message_obj = message_from_string(mail_text.strip())
        subject = message_obj['Subject']
        m_to = message_obj['To']
        m_from = message_obj['From']
        msg_type = message_obj.get('Content-Type', 'text/plain')

        host = getToolByName(portal, 'MailHost')
        try:
            host.send(mail_text, m_to, m_from, subject=subject,
                      charset=encoding, immediate=True,
                      msg_type=msg_type)
        except SMTPRecipientsRefused:
            # Don't disclose email address on failure
            raise SMTPRecipientsRefused(
                _(u'Recipient address rejected by server.'))
        except SMTPException as e:
            raise(e)

    def encode_mail_header(self, text):
        """ Encodes text into correctly encoded email header """
        return Header(safe_unicode(text), 'utf-8')

    def encoded_mail_sender(self):
        """ returns encoded version of Portal name <portal_email> """
        registry = getUtility(IRegistry)
        mail_settings = registry.forInterface(IMailSchema, prefix="plone")
        from_ = mail_settings.email_from_name
        mail = mail_settings.email_from_address
        return '"%s" <%s>' % (self.encode_mail_header(from_), mail)


@implementer(ILoginHelpForm)
class LoginHelpForm(form.EditForm):
    ''' Implementation of the login help form '''

    subforms = []

    id = 'LoginHelpForm'
    label = _(u'heading_login_form_help', default=u'Need Help?')

    ignoreContext = True

    def render(self):
        return self.index()

    def can_reset_password(self):
        # TODO: Actually check that the site allows reseting password
        return True

    def can_retrieve_username(self):
        # TODO: Actually check that the site allows retrieving the username
        return True

    def update(self):
        subforms = []
        # XXX: Not really sure how to handle the action and enctype vars
        if self.can_reset_password():
            form = RequestResetPassword(None, self.request)
            form.update()
            subforms.append(form)
        if not self.use_email_as_login() and self.can_retrieve_username():
            form = RequestUsername(None, self.request)
            form.update()
            subforms.append(form)

        self.subforms = subforms
        super(LoginHelpForm, self).update()

    def use_email_as_login(self):
        registry = getUtility(IRegistry)
        security_settings = registry.forInterface(
            ISecuritySchema, prefix='plone')
        return security_settings.use_email_as_login



# -*- coding: utf-8 -*-
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.login.testing import PLONE_LOGIN_FUNCTIONAL_TESTING
from plone.testing.z2 import Browser

import transaction
import unittest


class TestLoginLogout(unittest.TestCase):

    layer = PLONE_LOGIN_FUNCTIONAL_TESTING

    def setUp(self):
        self.browser = Browser(self.layer['app'])

    def test_login_with_bad_credentials(self):
        self.browser.open('http://nohost/plone/login')
        self.browser.getLink('Log in').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/login')

        self.browser.getControl('Login Name').value = TEST_USER_NAME
        self.browser.getControl('Password').value = 'wrongpassword'
        self.browser.getControl('Log in').click()

        self.assertIn('Login failed', self.browser.contents)
        self.assertEqual(self.browser.url, 'http://nohost/plone/login')

    def test_login_with_correct_credentials(self):
        self.browser.open('http://nohost/plone/login')
        self.browser.getLink('Log in').click()
        self.assertEqual(self.browser.url, 'http://nohost/plone/login')

        self.browser.getControl('Login Name').value = TEST_USER_NAME
        self.browser.getControl('Password').value = TEST_USER_PASSWORD
        self.browser.getControl('Log in').click()

        self.assertIn('You are now logged in.', self.browser.contents)
        self.assertEqual(self.browser.url,
                         'http://nohost/plone',
                         'Successful login did not redirect to the homepage '
                         'when came_from was not defined.')

        # Now log out.
        self.browser.getLink('Log out').click()
        self.assertEqual(self.browser.url,
                         'http://nohost/plone',
                         'Successful logout did not redirect to the homepage.')

        self.assertIn('You have been logged out.',
                      self.browser.contents,
                      'Logout status message not displayed.')

    def test_login_with_user_defined_in_root_user_folder(self):
        """ A user defined in the root user folder should be able to log
            in into the site
        """
        self.layer['app'].acl_users.userFolderAddUser('rootuser',
                                                      'secret',
                                                      [],
                                                      [])
        transaction.commit()
        self.browser.open('http://nohost/plone/login')
        self.browser.getControl('Login Name').value = 'rootuser'
        self.browser.getControl('Password').value = 'secret'
        self.browser.getControl('Log in').click()
        self.assertIn('You are now logged in', self.browser.contents)

    def test_not_logged_in_and_not_authorized_shows_login_form(self):
        self.browser.open('http://nohost/plone/@@overview-controlpanel')
        self.assertTrue(self.browser.getControl('Login Name'))

    def test_insufficient_privileges_returned_when_logged_in_but_not_authorized(self):  # noqa
        self.browser.open('http://nohost/plone/login')
        self.browser.getControl('Login Name').value = TEST_USER_NAME
        self.browser.getControl('Password').value = TEST_USER_PASSWORD
        self.browser.getControl('Log in').click()
        self.assertIn('You are now logged in', self.browser.contents)

        self.browser.open('http://nohost/plone/@@overview-controlpanel')
        self.assertIn('Insufficient Privileges', self.browser.contents)



# -*- coding: utf-8 -*-
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import MOCK_MAILHOST_FIXTURE
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from zope.configuration import xmlconfig


class PloneloginLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import plone.login
        xmlconfig.file(
            'configure.zcml',
            plone.login,
            context=configurationContext
        )

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'plone.login:default')


PLONE_LOGIN_FIXTURE = PloneloginLayer()
PLONE_LOGIN_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_LOGIN_FIXTURE, MOCK_MAILHOST_FIXTURE),
    name='PloneloginLayer:Integration'
)
PLONE_LOGIN_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_LOGIN_FIXTURE, MOCK_MAILHOST_FIXTURE, z2.ZSERVER_FIXTURE),
    name='PloneloginLayer:Functional'
)



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


version = '1.0rc1'

long_description = (
    open('README.rst').read() +
    '\n' +
    open('CONTRIBUTORS.rst').read() +
    '\n' +
    open('CHANGES.rst').read() +
    '\n'
)

setup(
    name='plone.login',
    version=version,
    description='Plone Login System',
    long_description=long_description,
    # Get more strings from
    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Environment :: Web Environment',
        'Operating System :: OS Independent',
        'Framework :: Zope2',
        'Framework :: Plone',
        'Framework :: Plone :: 5.0',
        'Framework :: Plone :: 5.1',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
    ],
    keywords='cms plone login security',
    author='The Plone Foundation',
    author_email='plone@plone.org',
    url='https://github.com/plone/plone.login',
    license='gpl',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['plone', ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'plone.app.z3cform',
        'plone.schema',
        'plone.z3cform',
        'six',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            'plone.api',
        ]
    },
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """
)



# -*- coding: utf-8 -*-
from plone.login import MessageFactory as _
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import transaction_note
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from zope.component import getMultiAdapter
from zope.component import queryUtility
from zope.interface import implementer
from zope.interface import Interface


class ILoggedOutView(Interface):
    pass


class LogoutView(BrowserView):

    def __call__(self):
        mt = getToolByName(self.context, 'portal_membership')
        mt.logoutUser(self.request)
        transaction_note('Logged out')
        # Handle external logout requests from other portals
        next_ = self.request.get('next', None)
        portal_url = getToolByName(self.context, 'portal_url')
        if next_ is not None and portal_url.isURLInPortal(next_):
            target_url = next_
        else:
            target_url = self.request.URL1 + '/logged-out'

        registry = queryUtility(IRegistry)
        external_logout_url = registry['plone.external_logout_url']
        if external_logout_url:
            target_url = external_logout_url
        self.request.response.redirect(target_url)


@implementer(ILoggedOutView)
class LoggedOutView(BrowserView):

    def __call__(self):
        portal_state = getMultiAdapter(
            (self.context, self.request),
            name='plone_portal_state',
        )
        if portal_state.anonymous():
            IStatusMessage(
                self.request
            ).addStatusMessage(
                _(
                    u'statusmessage_logged_out',
                    default=u'You have been logged out.'
                ),
                'info',
            )
            self.request.response.redirect(
                portal_state.navigation_root_url()
            )
            return
        return self.index()
